__version__ = "0.9.22"
__copyright__ = """
(C) 2018-2021 Andrew Andrianov <andrew@ncrmnt.org>, STC "Module"
https://module.ru / https://github.com/RC-MODULE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

"""

